# coding=utf8
import cStringIO
import progress
import socket
import time
import ssl
import os


class SockFeed:
    def __init__(self, httpConnection, chuck=1024):
        self.socket = httpConnection.connect
        self.buffer = None
        self.chuck_size = chuck
        self.head = None
        self.header = {}
        self.http_code = 0
        self.data = ''
        self.progressed = 0
        self.total = 0
        self.disable_progress = False
        self.last_stamp = time.time()
        self.top_speed = 0
        self.chucked = False
        self.title = ''

        self.file_handle = None

    def __del__(self):
        if self.file_handle:
            self.file_handle.close()

    @progress.bar()
    def http_response(self, file_path='', skip_body=False):
        if file_path and not self.file_handle:
            file_index = 1
            path_choice = file_path
            while os.path.exists(path_choice):
                path_choice = '{}.{}'.format(file_path, file_index)
                file_index += 1

            self.file_handle = open(path_choice, 'wb')
            self.title = path_choice
            # print("\033[01;31m{}\033[00m Downloading".format(path_choice))
        # while True:
        data = self.socket.recv(self.chuck_size)
        temp = cStringIO.StringIO(data)
        if not data:
            self.progressed = self.total = 100
            return self.data

        if not self.head or not self.header:
            self.head = temp.readline()
            self.http_code = int(self.head.split(" ")[1])
            if not self.http_code == 200:
                self.total = self.progressed = 1
                if self.file_handle:
                    current_file_name = self.file_handle.name
                    self.file_handle.close()
                    os.remove(current_file_name)
                return False
            while True:
                partial = temp.readline()
                if not partial or partial == '\r\n':
                    if self.header.get("Content-Length"):
                        self.total = int(self.header.get("Content-Length"))
                    elif self.header.get("Transfer-Encoding") == 'chunked':
                        self.chucked = True
                        print("\033[01;31mchucked encoding is not supported here for now\033[00m")
                        assert not self.chucked
                    break
                index = partial.index(":")
                key = partial[0: index].strip()
                val = partial[index + 1:].strip()
                self.header[key] = val
            if skip_body:
                self.total = self.progressed = 1
                return self.header
            left = temp.read()

            if left:
                if self.file_handle:
                    self.file_handle.write(left)
                else:
                    self.data += left

                self.progressed += len(left)

        else:
            if self.file_handle:
                self.file_handle.write(data)
                self.progressed += len(data)
            else:
                self.data += data
                self.progressed += len(data)


class HTTPCons:
    def __init__(self):
        self.host = ''
        self.port = 0
        self.connect = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

    def https_init(self, host, port):
        context = ssl.SSLContext(ssl.PROTOCOL_TLSv1)
        context.verify_mode = ssl.CERT_REQUIRED
        context.check_hostname = True
        context.load_default_certs()
        self.connect = context.wrap_socket(self.connect, server_hostname=host)
        self.connect.settimeout(105)
        self.connect.connect((host, port))
        self.host = host
        self.port = port

    def http_init(self, host, port):
        self.connect.settimeout(100)
        self.connect.connect((host, port))
        self.host = host
        self.port = port

    def request(self, url, method='GET', headers=None, data=None):
        if '//' not in url:
            raise URLNotComplete(url, 'url protocol')
        index = url.index('//')
        ishttps = url[0: index - 1].lower() == 'https'
        host_url = url[index + 2:]
        if "/" not in host_url:
            host_url += '/'
        index = host_url.index("/")
        host_port = host_url[0: index]
        url = host_url[index:]
        port = None
        if ':' in host_port:
            split = host_port.split(":")
            host = split[0]
            port = int(split[1].split("/")[0])
        else:
            host = host_port

        if ishttps:
            if not port:
                port = 443
            self.https_init(host, port)
        else:
            if not port:
                port = 80
            self.http_init(host, port)
        self.__send(url, method, headers, post_data=data)

    def __send(self, href, method='GET', headers=None, post_data=None):
        data = """{method} {href} HTTP/1.1
{headers}
"""
        if not headers:
            head = """Host: {}\r\n""".format(self.host)
            head += "User-Agent: HELLFLAME"
        else:
            head = ""
            for i in headers:
                head += "{}: {}\r\n".format(i, headers[i])
            if 'Host' not in headers:
                head += """Host: {}\r\n""".format(self.host)
            if 'User-Agent' not in headers:
                head += "User-Agent: HELLFLAME\r\n"
        if method == 'POST':
            if data and type(data) == str:
                # upload for one time
                head += "Content-Length: {}\r\n\r\n".format(len(post_data))
                head += "{}".format(post_data)
            else:
                raise URLNotComplete(href, 'POST data')
        elif method == 'GET':
            if post_data:
                assert type(post_data) == dict
                if '?' not in href[-1]:
                    href += '?'

                for i in post_data:
                    href += '{}={}&'.format(i, post_data[i])
        head += "\r\n"
        data = data.format(method=method, href=href, headers=head)
        # print data, 'EOL'
        self.connect.sendall(data)

    def __del__(self):
        self.connect.close()


def unit_change(target):
    unit_list = ('B', 'KB', 'MB', 'GB', 'TB')
    index = 0
    target = float(target)
    while target > 1024:
        index += 1
        target /= 1024
    return "{} {}".format(round(target, 2), unit_list[index])


class URLNotComplete(Exception):
    def __init__(self, url, lack):
        self.url = url
        self.lack = lack

    def __str__(self):
        return "URL: {} missing {}".format(self.url, self.lack)



#!/usr/bin/env python
# coding=utf8
import os
import sys
import manager

__version__ = '1.2.4'

short = {
    '--check': '-c',
    '--list': '-l',
    '--key': '-k',
    '--space': '-s',
    '--r-space': '-rs',
    '--help': '-h',
    '--remove': '-r',
    '--download': '-d',
    '--private': '-p',
    '--link': '-i',
    '--version': '-v',
    '--rename': '-n',
    '--list-a': '-la'
    }

map_target = {
    '--check': '查看文件状态',
    '--list': '文件列表',
    '--key': '修改或查看access key，secret key',
    '--space': '修改或查看当前空间名',
    '--help': '帮助',
    '--remove': '删除云文件',
    '--download': '下载文件',
    '--private': '返回私有文件下载链接',
    '--link': '返回开放云空间文件下载链接',
    '--version': '当前版本号',
    '--rename': "重命名",
    '--r-space': "删除本地保存的空间名",
    '--list-a': "显示本地已知所有空间文件列表"
    }


def help_menu():
    print("\n七牛云存储 Qiniu Manager\n")
    print("Usage:")
    print("  qiniu <your file> [space]\t\t选择文件位置上传，指定空间名或使用默认空间名")
    print("  qiniu [option] <file name> [space]\t对云空间中文件进行操作")
    print("  qiniu [--key|-k] <access key> <secret key>\t设置密钥\n")
    for i in map_target:
        print("  {},{}\t{}".format(i, short[i], map_target[i]))
    print("\n\033[01;31m首次使用\033[00m请设置密钥对 qiniu [--key|-k] <access key> <secret key>")
    print("必要情况下请设置默认空间名")


def main():
    argv = sys.argv
    argv_len = len(argv)
    if argv_len == 1:
        help_menu()
    else:
        arg = argv[1:]
        qiniu = manager.Qiniu()
        if argv_len == 2:
            if arg[0] in ('-v', '--version'):
                print("Qiniu Manager {}".format(__version__))
                # print("Qiniu SDK {}".format(manager.sdk_version))
            elif arg[0] in ('-h', '--help'):
                help_menu()

            elif arg[0] in ('-rs', '--r-space'):
                print("请手动指定将要从本地删除的空间名")

            elif arg[0] in ('-la', '--list-a'):
                qiniu.list_all()

            elif arg[0] in ('-l', '--list'):
                qiniu.list()

            elif arg[0] in ('-s', '--space'):
                data = qiniu.config.get_default_space()
                space_list = qiniu.config.get_space_list()
                print("Known Space Names Below\n")
                for i in space_list:
                    print("{} {}\t{}".format(space_list.index(i) + 1, i[0], i[1]))
                if data:
                    print ('\ndefault space name:\t\033[01;31m{}\033[00m'.format(data[0]))

            elif arg[0] in ('-k', '--key'):
                data = qiniu.config.access_list(include_discard=False)
                if data:
                    for i in data:
                        print ("{} {} {}".format(data.index(i) + 1, i[1], i[2]))
                else:
                    print("no access secret key pair found")

            else:
                target = arg[0]
                if os.path.exists(target):
                    qiniu.upload(target)
                    if qiniu.state:
                        print("\033[01;31m{}\033[00m upload in \033[01;32m{}\033[00m @\033[01;32m{}\033[00m"
                              .format(
                               os.path.basename(target),
                               qiniu.config.get_default_space()[0],
                               qiniu.avg_speed
                              ))
                    else:
                        print("\033[01;31m{}\033[00m uploaded \033[01;31mFailed\033[00m !!".
                              format(os.path.basename(target)))
                else:
                    print("\033[01;31m{}\033[00m not exist, use '-h' to help".format(target))
        elif argv_len == 3:
            if arg[0] in ('-c', '--check'):
                qiniu.check(arg[1])

            elif arg[0] in ('-l', '--list'):
                qiniu.list(arg[1])

            elif arg[0] in ('-s', '--space'):
                qiniu.config.set_space(arg[1])
                print("space \033[01;31m{}\033[00m is added and as the default space now".format(arg[1]))

            elif arg[0] in ('-rs', '--r-space'):
                qiniu.config.remove_space(arg[1])
                print("space \033[01;31m{}\033[00m is removed from local database".format(arg[1]))

            elif arg[0] in ('-r', '--remove'):
                qiniu.remove(arg[1])

            elif arg[0] in ('-d', '--download'):
                qiniu.download(arg[1])

            elif arg[0] in ('-p', '--private'):
                print(qiniu.private_download_link(arg[1]))

            elif arg[0] in ('-i', '--link'):
                print(qiniu.regular_download_link(arg[1]))

            else:
                target = arg[0]
                if os.path.exists(target):
                    if qiniu.config.get_space(arg[1]):
                        qiniu.upload(target, arg[1])
                        if qiniu.state:
                            print("\033[01;31m{}\033[00m upload in \033[01;32m{}\033[00m @\033[01;32m{}\033[00m".
                                  format(os.path.basename(target), arg[1], qiniu.avg_speed))
                        else:
                            print("\033[01;31m{}\033[00m upload FAILED, check the space name or network".format(
                                os.path.basename(target)))
                    else:
                        print("space {} not exist, use qiniu -s {} to add it".format(arg[1], arg[1]))
                else:
                    print("{} not exist, use qiniu -h for help".format(target))
        elif argv_len == 4:
            if arg[0] in ('-k', '--key'):
                qiniu.config.add_access(arg[1], arg[2])
                print("new key pair add")
                print("access:\t{}".format(arg[1]))
                print("secret:\t{}".format(arg[2]))

            elif arg[0] in ('-c', '--check'):
                qiniu.check(arg[1], arg[2])

            elif arg[0] in ('-r', '--remove'):
                qiniu.remove(arg[1], arg[2])

            elif arg[0] in ('-d', '--download'):
                qiniu.download(arg[1], arg[2])

            elif arg[0] in ('-p', '--private'):
                print(qiniu.private_download_link(arg[1], arg[2]))

            elif arg[0] in ('-i', '--link'):
                print(qiniu.regular_download_link(arg[1], arg[2]))

            elif arg[0] in ('-s', '--space'):
                qiniu.config.set_space(arg[1], arg[2])
                print("space \033[01;31m{}\033[00m now has the alias domain \033[01;31m{}\033[00m".format(arg[1],
                                                                                                          arg[2]))

            elif arg[0] in ('-n', '--rename'):
                qiniu.rename(arg[1], arg[2])

            else:
                help_menu()
        elif argv_len == 5:
            if arg[0] in ('-n', '--rename'):
                qiniu.rename(arg[1], arg[2], arg[3])

            else:
                help_menu()
        else:
            help_menu()


if __name__ == '__main__':
    main()





# coding=utf8
import sys
from setuptools import setup
reload(sys)
sys.setdefaultencoding('utf8')
__author__ = 'hellflame'
__version__ = '1.2.4'

setup(
    name='qiniuManager',
    version=__version__,
    keywords=('qiniu', 'qiniu manager', 'terminal command'),
    description="终端管理七牛云空间",
    license="MIT",
    author=__author__,
    author_email='hellflamedly@gmail.com',
    url='https://github.com/hellflame/qiniu_manager',
    packages=[
        'qiniuManager'
    ],
    platforms="UNIX like",
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        "Environment :: Console",
        "Operating System :: MacOS",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: POSIX :: Linux",
        'Programming Language :: Python :: 2.7'
    ],
    entry_points={
        'console_scripts': [
            'qiniu=qiniuManager.run:main'
        ]
    }
)

